/*
    IR Messenger - Infrared chat on TI89 calculators
    Copyright (C) 2003-2004  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _NO_INCLUDE_PATCH

#include "config.h"

#include <alloc.h>
#include <compat.h>
#include <dialogs.h>
#include <graph.h>
#include <kbd.h>
#include <statline.h>
#include <string.h>
#include <system.h>

#include "comm_lowlevel.h"
#include "comm.h"
#include "locale.h"
#include "log.h"
#include "screen.h"
#include "icon.h"

SCR_RECT MessageArea, EditorArea;

char EditorText[MESSAGE_LENGTH+1];

void clear_screen() {
  ClrScr();
  ST_helpMsg(""); /* otherwise, ST_eraseHelp() does not redraw the status line */
  ST_eraseHelp();
  DrawLine(0, LCD_HEIGHT-7, LCD_WIDTH, LCD_HEIGHT-7, A_NORMAL);
  DrawLine(0, MessageArea.xy.y1+1, MessageArea.xy.x1, MessageArea.xy.y1+1, A_NORMAL);
  EditorText[0] = 0;
  FontSetSys(F_6x8);
}

void hardware_probe() {
  while(!hwtest()) {
    if (DlgMessage(HW_PROBLEM, HW_NODETECT_RETRY, BT_YES, BT_NO) == KEY_ESC) {
      chatmode = FALSE;
      ST_busy(ST_NORMAL);
      return;
    }
    ST_busy(ST_NORMAL);
  }
}

void print_message(const char *message) {
  ScrRectScroll(&MessageArea, ScrRect, 9, A_REVERSE);
  DrawStr(0, MessageArea.xy.y1-9, message, A_NORMAL);
}

void received_message(const char *message) {
  char buffer[MESSAGE_LENGTH+3];
  if(!screen_on) PortSet(VirtualScreen, 239, 127);
  log_received_message(message);
  buffer[0] = 25;
  buffer[1] = ' ';
  memcpy(&buffer[2], message, MESSAGE_LENGTH+1);
  FontSetSys(F_6x8);
  print_message(buffer);
  if(!screen_on) { PortRestore(); ST_helpMsg(buffer); }
}

void editor_insert(const char *s) {
  if(strlen(EditorText)+strlen(s) > MESSAGE_LENGTH) return;
  strcat(EditorText, s);
  DrawStr(0, EditorArea.xy.y0+1, EditorText, A_REPLACE);
}

void editor_clear() {
  ScrRectFill(&EditorArea, ScrRect, A_REVERSE);
  memset(EditorText, 0, MESSAGE_LENGTH+1);
}

void editor_backspace() {
  unsigned long i;
  i = strlen(EditorText);
  if(i==0) return;
  i--;
  EditorText[i] = 0;
  ScrRectFill(&(SCR_RECT){{EditorArea.xy.x0+(i*6), EditorArea.xy.y0, EditorArea.xy.x0+((i+1)*6)+1, EditorArea.xy.y0+8}}, ScrRect, A_REVERSE);
}

void editor_send_message() {
  char buffer[MESSAGE_LENGTH+3];
  ST_helpMsg(SENDING_MESSAGE);
  if(send_message(EditorText)) {
    log_sent_message(EditorText);
    memset(buffer, 0, MESSAGE_LENGTH+3);
    buffer[0] = 26;
    buffer[1] = ' ';
    memcpy(&buffer[2], EditorText, MESSAGE_LENGTH+1);
    print_message(buffer);
    editor_clear();
  } else {
    DlgMessage(COMM_ERROR, MESSAGE_NOT_RECEIVED, BT_OK, BT_NONE);
    ST_busy(ST_NORMAL);
  }
  ST_eraseHelp();
}

void chatroom_main_loop() {
  void *kbq;
  unsigned short key;
  char buf[2];
  HANDLE h;
  unsigned long len;
  char buffer2[MESSAGE_LENGTH+1];

  kbq = kbd_queue();

  OSFreeTimer(USER_TIMER);
  OSRegisterTimer(USER_TIMER, 5);
  while(chatmode) {
    if(!OSdequeue(&key, kbq)) {
      ST_eraseHelp();
      switch(key) {
        case KEY_CLEAR:
          if(EditorText[0] == 0) clear_screen(); else editor_clear();
          break;
        case KEY_BACKSPACE:
          editor_backspace();
          break;
        case KEY_ENTER:
          editor_send_message();
          if(!chatmode) {
            close_log();
            uart_free();
            return;
          }
          FontSetSys(F_6x8);
          OSFreeTimer(USER_TIMER);
          OSRegisterTimer(USER_TIMER, 5);
          break;
        case KEY_ESC:
          close_log();
          DlgMessage(END_CHATTING, HW_DISCONNECT, BT_OK, BT_NONE);
          ST_busy(ST_NORMAL);
          uart_free();
          chatmode = FALSE;
          return;
        case HOTKEY:
          return;
        case KEY_DIAMOND+KEY_ESC:
          CB_fetchTEXT(&h, &len);
          if(len > MESSAGE_LENGTH) break;
          memset(buffer2, 0, MESSAGE_LENGTH+1);
          memcpy(buffer2, HeapDeref(h), len);
          editor_insert(buffer2);
          break;
        case KEY_DIAMOND+'/':
          editor_insert("!");
          break;
        case KEY_DIAMOND+0x16:
          editor_insert("@");
          break;
        case KEY_DIAMOND+'1':
          editor_insert(" :-)");
          break;
        case KEY_DIAMOND+'2':
          editor_insert(" :-(");
          break;
        case KEY_DIAMOND+'3':
          editor_insert(" ;-)");
          break;
        case KEY_DIAMOND+'4':
          editor_insert(" :-|");
          break;
        case KEY_DIAMOND+'5':
          editor_insert(" :-P");
          break;
        case KEY_DIAMOND+'6':
          editor_insert(" :o)");
          break;
        case KEY_DIAMOND+'7':
          editor_insert(" :-D");
          break;
        default:
          if(((key >= 'a') && (key <= 'z')) || ((key >= 'A') && (key<= 'Z'))
            || ((key >= '0') && (key <= '9')) || (key == '=') || (key == '(')
            || (key == ')') || (key == ')') || (key == ',') || (key == '^')
            || (key == '|') || (key == '.') || (key == '/') || (key == '*')
            || (key == '+') || (key == '-') || (key == ' ') || (key == ':')
            || (key == '"') || (key == '{') || (key == '}') || (key == '[')
            || (key == ']') || (key == ';') || (key == '<') || (key == '>')
            || (key == '\'') || (key == '\\')) {
              buf[0] = key;
              buf[1] = 0;
              editor_insert(buf);
          }
          break;
      }
    }
    idle();
    if(OSTimerExpired(USER_TIMER)) ScrRectFill(&(SCR_RECT){{EditorArea.xy.x0+strlen(EditorText)*6, EditorArea.xy.y0+1, EditorArea.xy.x0+strlen(EditorText)*6+1, EditorArea.xy.y1}}, ScrRect, A_XOR);
    check_incoming();
  }
}

void chatroom_start() {
  if(!chatmode) {
    uart_init();
    DlgMessage(START_CHATTING, HW_CONNECT, BT_OK, BT_NONE);
    ST_busy(ST_NORMAL);
    chatmode = TRUE;
    hardware_probe();
    if(!chatmode) {
      uart_free();
      return;
    }

    MessageArea.xy.x0 = 0;
    MessageArea.xy.y0 = 0;
    MessageArea.xy.x1 = LCD_WIDTH;
    MessageArea.xy.y1 = LCD_HEIGHT-7-11;

    EditorArea.xy.x0 = 0;
    EditorArea.xy.y0 = MessageArea.xy.y1+2;
    EditorArea.xy.x1 = MessageArea.xy.x1;
    EditorArea.xy.y1 = LCD_HEIGHT-7;

    clear_screen();
    BitmapPut(64, 0, &icon, ScrRect, A_NORMAL);
    FontSetSys(F_4x6);
    DrawStr(0, 32, CHATROOM_MSG1, A_NORMAL);
    DrawStr(0, 39, CHATROOM_MSG2, A_NORMAL);
    DrawStr(0, 46, CHATROOM_MSG3, A_NORMAL);
    DrawStr(0, 53, CHATROOM_MSG4, A_NORMAL);
    DrawStr(0, 60, CHATROOM_MSG5, A_NORMAL);    
    FontSetSys(F_6x8);
    memset(EditorText, 0, MESSAGE_LENGTH+1);

    open_log();
    chatroom_main_loop();
  } else
  {
    ST_helpMsg("");
    ST_eraseHelp();
    FontSetSys(F_6x8);
    chatroom_main_loop();
  }
}


